import React, { memo, useCallback, useMemo, useState, useEffect } from "react";
import { View, Text, Dimensions, StyleSheet } from "react-native";
import CodePush, { CodePushOptions } from "react-native-code-push";
import equals from "react-fast-compare";
import { CodePushProgressProps } from "./CodePushProgress.props";
const { width } = Dimensions.get("window");
/**
 *
 * Styles
 */

const styles = StyleSheet.create({
  loaderContainer: {
    position: "absolute",
    bottom: 0,
    left: 0,
    right: 0,
  },
  progressContainer: {
    position: "absolute",
    bottom: 0,
    left: 0,
    height: 15,
    zIndex: 2,
    backgroundColor: "#000",
    justifyContent: "center",
    alignItems: "center",
  },
  progressText: {
    position: "absolute",
    bottom: 20,
    left: 0,
    right: 0,
    zIndex: 1,
    textAlign: "center",
    color: "#fff",
    fontSize: 14,
    height: 20,
  },
});

/**
 * React.FunctionComponent for your hook(s) needs
 *
 * Component description here for TypeScript tips.
 */

const CodePushProgressBarComponent = (props: CodePushProgressProps) => {
  const {
    onDone = () => {},
    onStatusChange = () => {},
    codePushOptions = {
      checkFrequency: CodePush.CheckFrequency.ON_APP_START,
      installMode: CodePush.InstallMode.IMMEDIATE,
      mandatoryInstallMode: CodePush.InstallMode.IMMEDIATE,
    },
    customLoaderContainer = {},
    customProgressContainer = {},
    customProgressText = {},
    statusTextOptions = {
      CHECKING_FOR_UPDATE: "CodePush.sync Checking for update",
      DOWNLOADING_PACKAGE: "CodePush.sync Downloading package",
      AWAITING_USER_ACTION: "CodePush.sync Awaiting user action",
      INSTALLING_UPDATE: "CodePush.sync Installing update",
      UP_TO_DATE: "CodePush.sync App up to date.",
      UPDATE_IGNORED: "CodePush.sync Update cancelled by user",
      UPDATE_INSTALLED:
        "CodePush.sync Update installed and will be applied on restart.",
      UNKNOWN_ERROR: "CodePush.sync An unknown error occurred",
    },
  } = props;

  const [isLoading, setIsLoading] = useState<boolean>(false);
  const [downloadProgress, setDownloadProgress] = useState<number>(0);
  const [codepushStatus, setCodepushStatus] = useState<string>("");
  const [currentProgress, setCurrentProgress] = useState<number>(0);

  useEffect(() => {
    CodePush.sync(
      codePushOptions,
      (syncStatus) => {
        onStatusChange(syncStatus);
        switch (syncStatus) {
          case CodePush.SyncStatus.CHECKING_FOR_UPDATE:
            setCodepushStatus(statusTextOptions.CHECKING_FOR_UPDATE);
            console.log(statusTextOptions.CHECKING_FOR_UPDATE);
            break;
          case CodePush.SyncStatus.DOWNLOADING_PACKAGE:
            setCodepushStatus(statusTextOptions.DOWNLOADING_PACKAGE);
            console.log(statusTextOptions.DOWNLOADING_PACKAGE);
            break;
          case CodePush.SyncStatus.AWAITING_USER_ACTION:
            setCodepushStatus(statusTextOptions.AWAITING_USER_ACTION);
            console.log(statusTextOptions.AWAITING_USER_ACTION);
            break;
          case CodePush.SyncStatus.INSTALLING_UPDATE:
            setCodepushStatus(statusTextOptions.INSTALLING_UPDATE);
            console.log(statusTextOptions.INSTALLING_UPDATE);
            break;
          case CodePush.SyncStatus.UP_TO_DATE:
            setCodepushStatus(statusTextOptions.UP_TO_DATE);
            console.log(statusTextOptions.UP_TO_DATE);
            codePushSyncCompleted();
            break;
          case CodePush.SyncStatus.UPDATE_IGNORED:
            setCodepushStatus(statusTextOptions.UPDATE_IGNORED);
            console.log(statusTextOptions.UPDATE_IGNORED);
            codePushSyncCompleted();
            break;
          case CodePush.SyncStatus.UPDATE_INSTALLED:
            setCodepushStatus(statusTextOptions.UPDATE_INSTALLED);
            console.log(statusTextOptions.UPDATE_INSTALLED);
            break;
          case CodePush.SyncStatus.UNKNOWN_ERROR:
            setCodepushStatus(statusTextOptions.UNKNOWN_ERROR);
            console.log(statusTextOptions.UNKNOWN_ERROR);
            setIsLoading(false);
            codePushSyncCompleted();
            break;
        }
      },
      (progress) => codePushDownloadDidProgress(progress)
    );
  }, []);

  const codePushDownloadDidProgress = (progress: any) => {
    const { receivedBytes, totalBytes } = progress;
    console.log("codepush progress", progress);
    const current = parseFloat(receivedBytes) / parseFloat(totalBytes);
    setDownloadProgress(current);
  };

  const codePushSyncCompleted = () => {
    onDone();
  };
  const displayProgress = Math.round(downloadProgress * 100);

  return (
    <View style={[styles.loaderContainer, customLoaderContainer]}>
      <View
        style={[
          styles.progressContainer,
          customProgressContainer,
          { width: width * downloadProgress },
        ]}
      />
      <Text style={[styles.progressText, customProgressText]}>
        {downloadProgress > 0
          ? `${codepushStatus} ${displayProgress}%`
          : codepushStatus}
      </Text>
    </View>
  );
};

export const CodePushProgressBar = memo(CodePushProgressBarComponent, equals);
